function [x_opt] = conj_grad(x0,a)
%
%%%%%%%%%%%%%%%%%%%%%%%%%WARNING%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Before running 'conj_grad', 'A' has to be defined and set to be
% global. We also need to define the global variable counter to take 
% into account the number of matrix vector multiplications when this
% function is used with 'newtrust.m'. The following instruction need
% be done
%
% >> clear A counter
% >> global A D counter
% >> A = [.....];
% >> counter = 0;
%
% If 'newtrust' is the function we are interested in and 'conj_grad'
% might be called, then before using 'newtrust', do the following 
%
% >> clear A D counter
% >> global A D counter
% >> A = [...]
% (Note: counter now doesn't need to be initialized as it will be
%  in 'newtrust') 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This function solves the problem
%
% min (1/2)*x^tAx - a^tx
%
% and A is assumed positive definite
%
% Call: [x_opt,f_opt] = conj_grad(a)
%
% INPUT:    a     = a vector
%           x0    = a initial estimate for x_opt
%
% OUTPUT:   x_opt = the global minimum
%
% STOPPING CRITERIA: norm(A*x-a) < zerotol:= max(10^(-15),10^(-5)*norm(a));
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global A D counter

% Initialize variables

%zerotol = 1e-2;
zerotol = max(10^(-15),10^(-5)*norm(a));
%zerotol = norm(a)*min(0.1,norm(a)^(0.1));

x = x0; % x is the current iterate

J = A*x0-a; % d is the gradient of the current iterate

nJ = norm(J); % norm of the gradient of the current iterate

J_1 = J; % d will be the gradient of the previous iterate

nJ_1 = norm(J_1); % norm of the gradient of previous iterate

d_1 = 0; % previous step direction

counter = counter +2;  % counter counts the number of matrix vector 
                       % multiplications

while (nJ > zerotol)

  d = J + (nJ/nJ_1)^2*d_1; % current step direction

  d_1 = d;

  r = (J'*d)/(2*(d'*A*d)); % step lenght

  x_1 = x; % previous iterate is x_1

  J_1 = J; 

  nJ_1 = nJ;

  x = x - r*d;

  J = A*x-a;

  nJ = norm(J);

  counter = counter + 2;

end

x_opt = x;





