%%% muup must always correspond to bestxf??? bestfxf???
function [xopt,fopt,lopt,lambdamin,flag] = newtrust4b(a,s,dgaptol)
%%%%%%
% Call:  
%[xopt,fopt,lopt,lambdamin,flag] = newtrust4b(a,s,dgaptol)
%%%%%%
%
% solves: min      f(x) = x'Ax - 2a'x 
%      such that        x'x <= s^2.
%
%%%%%%%
%% files needed: Afun.m,Dfun.m,conj_grad.m, and this file newtrust4b.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%WARNING%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%                                                                          %
% BEFORE WORKING WITH NEWTRUST4 IN MATLAB, THE FOLLOWING INSTRUCTION HAS   %
% TO BE DONE:                                                              %
%                                                                          % 
%                            >> global A D counter                         %
%                                                                          %
% THIS TELLS MATLAB THAT BOTH A,D AND COUNTER ARE CONSIDERED GLOBAL IN THE % 
% PROGRAM                                                                  %  
%                                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%WARNING%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%      This code is available by anonymous ftp to orion.uwaterloo.ca
%        and over WWW   URL:   http://orion.uwaterloo.ca/~hwolkowi
%
%     More information on this algorithm can be found in the document
%     'newtrust_guide.ps'. In particular, the equivalent of this matlab
%     file is rewritten and more comments are included. A 'theory' section
%      also explains the different techniques used in the algorithm.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   The user can change the stopping tolerances, for the optimal value,
%      for feasibility, and the iteration count, below.
%   The user can also change the formulae for the various initial
%      bounds, below.
%
%  INPUT: 
%         A     ... symmetric matrix - assumed to be in sparse format
%                            global variable due to Lanczos calls
%         a     ... column vector - it is changed to full format below
%         s>0   ...   radius of trust region
%        dgaptol... duality gap tolerance - changes throughtout the trust
%                   algorithm.
%
% OUTPUT: 
%         lopt ... dual opt multiplier vector,
%         xopt ... opt. primal solution vector,
%         fopt ... value of objective function at a feasible point,
%              ...   value is < gap(1,1)+dgaptol if assigned a value;
%              ...   takes value 99999 if not assigned a value.
%       flag   ... a boolean variable to know if solution is in the interior 
%                  (flag ==2) and if the hard case (case 2) (or near hard case
%                  (case2)) occurs (flag ==1) and otherwise (flag == 0).
%    lambdamin ... minimum eigenvalue of A
%
% OTHER VARIABLES OF INTEREST:
%
%        iter  ... number of iterations - success occurred if < iterbnd
%        gap   ... gap, optimal value is in interval [gap(1,1),gap(1,2)]
%
%             on output, vector gap is:
%
%             gap=[mulow muup        % lower and upper bounds on mu^* (the optimum
%                                    % of the quadratic objective )
%             ptbad(1,3) ptgood(1,3) % lower and upper bounds on lambda^* (multiplier)
%             low high               % lower and upper bounds on t^*
%             tarn1f t0f]
%       tarn1f ... = cputime-tarn10;  save initial eigenvalue cputime for statistics
%          t0f ... = cputime - t0;  total computation time
%         topt ... right t-value to generate opt. solution;
%                     t=99999 means that lopt=0 used or 
%                     0 iterations with no optimal t,
%                       i.e. an unconstrained optimum was found.
%                     t is parameter in eigenvalue problem.
%
%    ptgood, ptbad variables: ... used for saving interpolation data
%                     bestxf  ... current best approximation to the x-optimum
%                     bestfxf  ... current best approximation to the optimum of f(x)  
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%GLOBAL VARIABLES%%%%%%%%%%%%%%%%%%%%%%%
%
global A D counter
sparse A D;
if not(issparse(A))| not(issparse(D)),
    disp(['A or D not in sparse format in newtrust4b']);
    keyboard
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%TOLERANCES%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tolerances - these can and should be changed by the user.
%
%    The tolerance for the duality gap is for significant figures,
%     i.e. it is a relative duality gap.
%     The significant figures of agreement are computed for
%       the duality gap as follows:
%      signif = max( -log_10  |primal-dual|/(|primal|+1)  , 0  )
%
%    The tolerances for ***LANCZOS** should be significantly higher
%         than those used here, e.g. 4 here and 7 for Lanczos
%
% Stopping criteria:  (i)   feasibility,      ||x|| <= (1+normtol)s
%            ???      (ii)  ????????,      dbestgap= (bestfxf-muup)/(|bestfxf|+1)
%            ???                                                <= 2*dgaptol
%                     (iii)  duality gap,      |gap(2)-gap(1)|/(|gap(2)|+1)
%                                                               <= dgaptol
%                     (iv) iteration count,  iter > iterbnd
%                     (v) interval of uncertainty for t is < zerotol
%      Stop when:  ( ((i) and (ii)) or (iii))  and ((iv) and (v))
%
%dgaptol                % duality gap for stopping criteria
normtol  =1e-9;         % allowable error in feasibility for stopping
iterbnd=30;             % upper bound on iterations
n = size( A,1);         % problem size
zerotol=(1e-4)/n;       % tolerance for zero for y(1) for hard case
                        % since y is norm 1, tolerance decreases with dim.
%%%%%%%%%%%%%%%%%%%%%%%%%%%END TOLERANCES%%%%%%%%%%%%
% 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%INITIALIZATION%%%%%%%%%%%%%%
t0=cputime;             % saved to get total cputime usage

% Initialize "counter". "counter" gives the number of matrix-vector
% multiplications
counter = 0;

% Initial estimates  -  

a=full(a);     %  set to full to avoid conflicts
na=norm(a);      % vector norm

tarn10=cputime; % We want to keep for statistical purpose the first eig calc
                % cputime

OPTIONS.issym = 1; % because we know the matrices A and D are symmetric
OPTIONS.disp = 0; % no display of the output for eig.m
%v=min(diag(A));
%d=diag(diag(A)+2*(abs(v)+1)*ones(n,1));
%v=d\a;
v=full(sum(A))';       % good/cheap estimate for sparse A ???
OPTIONS.v0 =v ; % initial eigenvector estimate

%%%%% adding this can accelerate the computations of the first eigenvalue
%lambda = min(eig(A));
%R = A;
%for i=1:n
%	R(i,i) = R(i,i) -lambda;
%end
%R = sparse(R);

%[va, lambda2] = eigs(R,1,0,OPTIONS);

%%%% you also need to put the next line as a comment

[va,lambda] = eigs('Afun',n,1,'SA',OPTIONS); % computes smallest
                   %eigenvalue (lambda) and corresponding eigenvector (va) for A

lambdamin = lambda; % save the min eigenvalue of A

flag = 0; %flag will change value if the almost hard case (case 2) occurs or if
          %the solution is in the interior.

tarn1f=cputime-tarn10;   % save initial eig calc cputime for statistics
disp(['elapsed time for first eig calc',num2str(tarn1f)]);

y=[1;va];     % initial estimate for the eigenvector of D(t^*)
alow=lambda;aup=lambda;

%two lines replaced by following 6 lines jan 10/00
%% lower estimate on mu*
%mulow = alow*s^2-2*na*s;  % lower estimate for optimal value

% lower estimate on mu*
if lambda<0
   mulow = alow*s^2-2*na*s;  % lower estimate for optimal value
else
   mulow = -2*na*s;
end

bestxf=(s/norm(va))*va; % current best feasible solution

% upper estimate on mu* -optimal value- uses feasible eigenvector 

% ensure va'*a is not negative
if (va'*a < 0)
   va = -va;
end

% upper estimate on mu*
if lambda > 0
  if (~((va'*a)/(lambda)>s))
     muup = -(va'*a)/lambda;
     bestxf = -((va'*a)/lambda)*va;
  else
     muup = s^2*lambda - 2*s*(va'*a);
     bestxf = s*va;
  end
else
  [muup,indice] = min([0,lambda*s^2-2*s*(va'*a)]);
  if indice == 1
    bestxf = zeros(size(a,1),1);
  else
    bestxf = s*va;
  end
end

bestfxf=muup; % current best opt value corresp to bestxf
mubnds=[mulow muup];    % keep history of bounds - latest at top
                        % This is only for debugging help.
if (lambda > 0)
    low = min(alow - na/s,-na*s);
else
    low = alow - na/s;  % low: always describes lower bound on t-gets updated
end

high = aup + na*s;  % high: ... upper bound on t-gets updated

%    End of initial estimates
%
%   More initialization of variables

gap=[mulow muup];         % duality gap - gets updated when a bound changes
ss1=s^2+1; 
D = [ 0 -a' 
     -a   A];           % augmented matrix

ystar = 1 / sqrt( s*s +1);      % this is value we are after for y0(t)
iter=0;         % iteration count
b = '      ';              % define blanks
%
% ptgood, ptbad variables: used for saving interpolation data on the
%  good (easy) and bad (hard) side of maximum of k(t).
% The latest data is put into row 1.
%   Only two rows are needed. The rest is saved for debugging help.
%    t y(1) lambda  is data saved.
ptgood=[]; 
ptbad=[];  

gy=[];by=[];    % The latest vectors y from the good and bad side resp.
side=0;        % indicator for consecutive t found 
                %  -i for i bad sides and +i for i  good sides
tslopes=[];    % Save slopes of kt on the [bad good] sides - for
%               debugging and for case where kt is almost piecewise linear.
kts=[];      % saving kt bounds and t value - for debugging
bestfxf=muup;         % for first while test below
t=99999;indexg=0;indexb=0;  % initialize in case of 0 iterations
tind=[];     % follow trail of t changes - for debugging
%
disp( 'iteration  - -   t  - -  relative duality gap - - feasibility gap');
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%NOTES on ALGORITHM%%%%%%%%%%%%%%
%   Following are the functions of interest:
%    h(lambda)  =  s^2lambda -a'(A-lambda I)^{-1} a;      maximize - concave
%    hp(lambda)  = s^2 -a'(A-lambda I)^{-2} a;solve = 0 - concave, nonincreasing
%    phi(lambda)  = 1/s-1/||(A-lambda I)^{-1} a||;solve = 0 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    k(t)  =  (s^2+1)lambda -t         maximize - concave
%    k'(t) = (s^2+1)y0^2(t) -1         solve = 0  - same sign as f(t)
%    psi(t)  = -1/y0(t) + sqrt(s^2+1)    solve = 0 - concave, nonincreasing
%    y0(t) = first component of y normalized   - nonincreasing, 1to0
%    lambda=lambda(t)    concave function
%
% The algorithm is based on maximizing k(t).
% We interpolate on psi(t) whenever possible.
%
% The general iteration interpolates psi(t) whenever
%   possible. Duality is used to calculate bounds, i.e. the function k(t) is
%   used when problems with the hard case occur.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%END INITIALIZATION%%%%%%%%%%%%%
%
%
%%%%%%%%%%%%%%%%%START GENERAL ITERATION%%%%%%%%%%%%%%
%

% Update the tolerances.
t1=1;    % feasibility tolerance using y(1)
t2= ( (bestfxf-muup)/(abs(bestfxf)+1) )   > 2*dgaptol; %%???why???
t3= ( (muup-mulow)/(abs(muup)+1) )   > dgaptol; % duality gap tolerance
t4=iter <=iterbnd; % iteration bound
t5=abs(high-low)/(abs(high)+1) > zerotol; %interval of uncertainty for t
 
%  check feasibility and iteration count and duality gap
while ( (t1&t2) | t3) & (t4 & t5),

  iter=iter+1; % one more iteration is done
  indexg=size(ptgood,1); % number of points on the good side
  indexb=size(ptbad,1); % number of points on the bad side

  %  Find a new point t.
  t=(low+high)/2;      % midpoint default
  tind=[tind 1];    % indicator for t value - for debugging

  if indexb > 0 & indexg > 0,
  % TRIANGLE INTERPOLATION

  % Always find an interpolant using triangle - with an associated upper bound.
     tlslope=ss1*ptbad(1,2)^2-1; % slope of k(t) at the last bad side point
     thslope=ss1*ptgood(1,2)^2-1;% slope of k(t) at the last good side point
     tslopes=[tlslope thslope
               tslopes];         % for debugging purposes - possible use
                                % when both sides are almost linear
     ktl=ss1*ptbad(1,3)-ptbad(1,1);  % value of k(t) at last bad side point
     kth=ss1*ptgood(1,3)-ptgood(1,1); % value of k(t)at last good side point
     mrhs=[  (tlslope*(-ptbad(1,1))+ktl)
               (thslope*(-ptgood(1,1))+kth)     ];   % rhs of interpolation
     ktt=[ 1  (-tlslope)
           1  (-thslope)];
     kttans=ktt\mrhs; % This finds the intersection of the two tangent lines at
                      % the last bad and good side points. The point of
                      % intersection is (kttans(2),kttans(1)). Hence, kttans(1)
                      % is an upper bound on mu^* and kttans(2) is an approximation
                      % for t^*
     ktup=kttans(1);
     tt=kttans(2);   % This is alternate interpol point using triangle
     if low <=tt&tt<=high,  % roundoff error problem check
%  low = tt can occur in hard case and indicates low is almost optimal.
%  An error in divide by zero can occur since iterate is repeated in
%     interpolation.
        t=tt; % new update for the approximation of t^*
        tind=[tind 2];     % indicator for t value - for debugging
     end
     if (ktup <=muup) & (ktup >= mulow),
      muup=ktup;           % new upper bound on optimal value
     end
  end     % if indexb > 0
%  End of triangle for new interpolant

 if side>0,   % last iterate was on the good side

  if  indexg>=2, % At least two t-iterates are on the good side    
   if (lambda<0) & (min(ptgood(1:2,2)) > (10^(side-1)*zerotol) ),   
           % two easy case points - prevent too many good side iterates
% INVERSE INTERPOLATION - interpolate on good side
% Let t(psi) be the inverse function of psi(t), then we linearly interpolate
% the value t(0), which is t^*, using the two points from the good side.
     tint=ptgood(1:2,1);
     yint=ptgood(1:2,2);
     f1=-1/yint(1)   + 1/ystar;
     f2=-1/yint(2)   + 1/ystar;
%  Slope on the good side should always guarantee f1-f2 not =0.
     slope=(tint(1)-tint(2))/(f1-f2);
     tt=tint(1) - f1* slope;    % linear inverse interpolation
     if low <=tt&tt<=high,     % roundoff error problem check
        t=tt;
        tind=[tind 3];     % indicator for t value - for debugging
     end
   elseif side>4,    % We need a point on the bad side - too many 
                          %iterates on good side
     t=.95*low+.05*high;
     tind=[tind 4];     % indicator for t value - for debugging
   end   % end of lambda < 0
  end   % end of if indexg>=2

 elseif side<0,   % last iterate was on the bad side
  if indexb >=2,   % at least two bad side points-
%INVERSE INTERPOLATION
% Let t(psi) be the inverse function of psi(t), then we linearly interpolate
% the value t(0), which is t^*, using the two points from the bad side. 

%?????  %interpolate from bad side if we have a good side point already.

     tint=ptbad(1:2,1);
     yint=ptbad(1:2,2);
     f1=-1/yint(1)   + 1/ystar;
     f2=-1/yint(2)   + 1/ystar;
% If f1=f2, then a t value has been repeated by e.g. t=low in triangle.
% This indicates that the optimal t is very close to t=low.
    if f2>f1,   % correct monotonicity occurred
     slope=(tint(1)-tint(2))/(f1-f2);
     tt=tint(1) - f1* slope;    % linear inverse interpolation
       if low<=tt&tt<(high-zerotol), 
                 % Use the new point - it is not too large
         t=tt;
         tind=[tind 5];     % indicator for t value - for debugging
       elseif indexg==0&side>-4,   % need a point on good side
         t=.95*high+.05*low;
         tind=[tind 6];     % indicator for t value - for debugging
       elseif indexg==0&side<-4,   % need a point on good side desperately
         t=high;
         tind=[tind 7];     % indicator for t value - for debugging
       end
    elseif side>-4,
     t=.95*low + .05*high;  % try to stay on bad side but not too long
     tind=[tind 8];     % indicator for t value - for debugging
    end
  end    %  end for indexb>=2
 end    % end for side check
disp(['side: ', num2str(side)]);
disp(['trail for t: ', num2str(tind)]);


%%%%%%%%%%%UPDATES%%%%%%%%%%%

%  A new point t has been found. Start updates.
%
% Recall t is the most recent update for t^*

D( 1, 1) = t;

tarn20=cputime;

OPTIONS.v0 = y; % y is the most recent estimate for the eigenvector of D(t^*)
[y,lambda] = eigs('Dfun',n+1,1,'SA',OPTIONS); % y and lambda are the new estimates
                                % for the eigenvector of D(t^*) and for lambda^*

tarn2f=cputime-tarn20;   % save initial Lanczos call cputime for statistics
disp(['elapsed time for second eig calc',num2str(tarn2f)]);

if y(1)<0,  % ensure the sign of the first component is positive
  y=-y;
end

%  multiplier sign check - to update lower bounds
if lambda > 0,   % A is known to be pos def - multiplier has wrong sign

   if y(1) >= ystar,
   % This means that y(2:n+1)/y(1) is the optimum solution for all points outside
   % the trust region of radius (1-y(1)^2)/y(1)^2 or on its boundary. If this 
   % radius is less than s^2, as it is the case here since this is equivalent to
   % y(1) >= ystar, then this means that the optimum we are looking for is inside
   % the trust region and A is positive semidefinite. We will then apply a conjugate
   % gradient method to find the optimum. Hence, y(2:n+1)/y(1) is feasible so
   % optimal with lambda=0.

       a=sparse(a);

       xopt = conj_grad(y(2:n+1)/y(1),a); % apply conjugate gradient method to 
                                          % find the optimum.

       gap  = -a'*xopt;   % optimal value since A xopt = a
       fopt=gap;
        lopt = 0; 
        topt = 99999;
        gap=[topt topt
             lopt lopt
             topt topt
               ];
        t0f=cputime-t0;
        flag = 2; % To know answer is the unconstrained minimum
        disp(['elapsed time: ', num2str(t0f)]);
        return;    %OPTIMALITY - PROBLEM IS SOLVED
    end
else     % the sign of the multiplier is correct
         %   so apply duality theory
    kt=ss1*lambda-t;           % value of k(t), dual functional. This gives
                               % a lower bound on mu^*
    kts=[t lambda kt
            kts];
    if (kt>mulow)&(kt<=muup),
      mulow=max(mulow,kt);           % new lower bound on optimal value
    end
%
end        % end of if lambda > 0 loop
%%  end multiplier sign check

%
if y(1) >= ystar,        % bad side
     low = t;           % new lower bound on t^*
     ptbad=[t y(1) lambda
           ptbad];       % Keep points for first interpolation.
    side=min(-1,side-1);  % consecutive bad sides
    by=y;
%%
%%
%%%%%%%%%%%%%
%% Begin bound for PSDP from bad side - approx. negative curvature
%  direction and move to the boundary.
   if indexg>0,
    if  ptgood(1,3) <= 0,   % multiplier has correct sign on good side
     normx2=(1-y(1)^2)/(y(1)^2); % square of norm of current soln.
     xf=y(2:(n+1))/y(1);   % current soln. inside trust region

% In the hard case (case 2), we use MoSo formula page 558 - however
% with negative eigenvalue and eigenvector, to move to take a step to
% the boundary. In the easy case and hard case (case 1), we take a step
% to the boundary by taking the solution between the line joining the 
% xf and the x-point - gy(2:n+1)/gy(1,1) -  outside the trust region
% associate with the last good side t-point.

	 if (gy(1) < zerotol)
		% This is the almost hard case (case 2)
		z = gy(2:n+1);
     	        normz=norm(z); % normz should be almost 1   

	 else
		% This is the easy case or almost hard case (case 1)
	 	z = gy(2:n+1)/gy(1,1)-by(2:n+1)/by(1,1);
	  	normz=norm(z);

	 end


     xyprod=(z'*xf)/normz;  % normalized inner product
     ndiff=s^2-normx2;
     sxy=sign(xyprod);
     if sxy==0,
        sxy=1;
     end
     tau=ndiff/(xyprod+sxy*sqrt(xyprod^2+ndiff) );
     xnew=xf+(tau/normz)*z;  % xnew is the point obtained by moving to the
                             % boundary
     fxnew=xnew'*A*xnew-2*xnew'*a;
   % get new upper bound using the feasible xnew - need to allow almost
      %  best xnew as the bestxf
     if fxnew<bestfxf,
        bestxf=xnew;
        bestfxf=fxnew;
     end
     if (fxnew<muup) & (fxnew >=mulow),
        muup=fxnew;
     end

    end      % end if <= 0 line mult. check
   end        %end of if indexg line
   %%%%%%%%%% end of PSDP bound
%     
%%%%%%%%%%%%%%%%%%%
else                    %  current point is on good side
     high = t;       % we have new upper bound on t
     ptgood=[t y(1) lambda 
             ptgood   ] ;
    side=max(1,side+1);  % consecutive bad sides
    gy=y;
  if  ptgood(1,3) <= 0,   % current multiplier has correct sign
    if indexb > 0,
%% begin type 1 bound for PSDP from the good side
     normx2=(1-by(1)^2)/(by(1)^2);  % use last bad side point
     xf=by(2:(n+1))/by(1);

% In the hard case (case 2), we use MoSo formula page 558 - however
% with negative eigenvalue and eigenvector, to move to take a step to
% the boundary. In the easy case and hard case (case 1), we take a step
% to the boundary by taking the solution between the line joining the 
% xf and the x-point - gy(2:n+1)/gy(1,1) -  outside the trust region
% associate with the last good side t-point.

	 if (gy(1) < zerotol)
		% This is the almost hard case (case 2)
		z = gy(2:n+1);
                normz=norm(z); %normz  
	 else
		% This is the easy case or the almost hard case (case1)
	 	z = gy(2:n+1)/gy(1,1)-by(2:n+1)/by(1,1);
	  	normz=norm(z);
	 end


     xyprod=(z'*xf)/normz;  % normalized inner product
     ndiff=s^2-normx2;
     sxy=sign(xyprod);
     if sxy==0,
        sxy=1;
     end
     tau=ndiff/(xyprod+sxy*sqrt(xyprod^2+ndiff) );
     xnew=xf+(tau/normz)*z;  %
     fxnew=xnew'*A*xnew-2*xnew'*a;
   % get new upper bound using the feasible xnew - need to allow almost
      %  best xnew as the bestxf
     if fxnew<bestfxf,
        bestxf=xnew;
        bestfxf=fxnew;
     end
     if (fxnew<muup) & (fxnew >=mulow),
        muup=fxnew;
     end

    else      % else of indexb>0 line
% Need upper bound as well in case triangle was not used.
%
%% begin type 2 bound for PSDP
       xf=y(2:(n+1));    % norm is sqrt of 1-y_0^2
       xnew=(s/sqrt(1-y(1)^2))*xf; % xnew is the point obtained by moving to the
                                   % boundary
       fxnew=xnew'*A*xnew-2*xnew'*a;
        % get new upper bound using the feasible xnew
       if (fxnew<muup) & (fxnew >=mulow),
          bestxf=xnew;
          bestfxf=fxnew;
          muup=fxnew;
       end
%% reached end type 2 bound for PSDP
   end     % ind of if indexb > 0 line
  end      % end if <= 0 line - multiplier sign check
end        % end of if bad side
%%%%%%%%%%%%
%% save new bounds and gap
if mulow > mubnds(1,1) | muup < mubnds(1,2),
  mubnds= [ mulow muup
      mubnds ];
  gap=[mulow muup];    % duality gap - gets updated when a bound changes
end
%
% update tolerances
t1=abs( 1-y(1)^2*(1+s^2)) > (normtol*y(1)^2*s^2 ); % Feasibility criteria
dbestgap= (bestfxf-muup)/(abs(bestfxf)+1);
t2= dbestgap   > 2*dgaptol; %??????
dgap=  abs(muup-mulow)/(abs(muup)+1) ;
t3= dgap >  dgaptol; % Duality gap criteria
t4=iter <=iterbnd; % iteration bound criteria
t5=abs(high-low)/(abs(high)+1) > zerotol; % gap for t^* criteria
txt=[int2str(iter),b,b,b,num2str(t),b,b,num2str(dgap),b,b,num2str(dbestgap)];
disp( txt);

end         % end of while for gap and iteration check
%
%%%%%%%%%%%%%%%%%%%%%%%%%END GENERAL ITERATION%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%OUTPUT DATA%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fopt=(muup+mulow)/2;
if ~t2,           % bestxf is almost feasible
   xopt=bestxf;
   fopt=bestfxf;
else
   xopt = y(2:n+1)/y(1);
end
  topt=t;lopt=lambda;
  if indexg ==0,
       ptgood=[99999 99999 99999];
  end
  if indexb ==0,
       ptbad=[-99999 -99999 -99999];
  end
  t0f=cputime-t0;
  gap=[gap                    % lower and upper bound for mu^*
       ptbad(1,3) ptgood(1,3) % lower and upper bound for lambda^*
       low high               % lower and upper bound for t^*
       tarn1f t0f ];          % tarn1f is elapsed time for first eigs call
                              % t0f is total computation time

disp(['elapsed time: ', num2str(t0f)]); % t0f is total computation time
disp(['number of matrix-vector multiplications:', num2str(counter)])

if (abs(lambdamin - lopt)/(abs(lambdamin)+1) < 1e-2)
  flag = 1;
end
