function F = objective(x,prob)

% This function returns the objective value F(x) of F associated with problem
% number "prob"
%
% Here are the problems and their related number
%
% 1 = Broyden banded function BRYBND (31) in [1]
% 2 = Generalized Rosenbrock function GENROSE [2]
% 21 = GENROSE, but a scalar of 1000 is used instead in the function
% 22 = A function close to GENROSE (no reference)
% 23 = generalization of the banana function (no reference)
% 3 = Optimal Sensor Placement SENSORS [3]
% 4 = Extended Powell Singular Function (22) in [1]
% 5 = NONDIA in [4]
% 6 = TRIDIA in [4]
% 7 = EXTROS in [4]
% 8 = Discrete Boundary Value Function (28) in [1]
% 9 = Broyden Tridiagonal Function (30) in [1]
% 10= Watson Function (20) in [1]
% 11= Brown Almost-Linear Function (27) in [1]
%
% Note that for the last 2 problems, 'm' can be variable. It is fixed to 2*n
% in the code right now.
%
% 12= Linear Function-full rank (32) in [1]
% 13= Linear function-rank 1 (33) in [1]
% 14= Linear function-rank 1 with zero columns and rows (34) in [1]
% 15= A function I created so the hard case (case 2) would occur initially
% 16 & 17 = two other function where the hard case occurs
%
%
% [1] = J.J. More, B.S. Garbow and K.E. Hillstrom, Testing unconstrained
%       optimization software, 7(1981), no 1, pp. 17-41.
%
% [2] = S. Nash, Newton-type minimization via the Lanczos process, SIAM journal
%       on numerical analysis, 21, 1884, pp. 770-788.
%
% [3] = H. Zhang, X. Wang, Optimal Sensor placement, SIAM Review, vol. 35,
%       pp. 641, 1993.
%
% [4] = G. Liu and J. Nocedal, Test results of two limited memory methods for 
%       large scale optimization, Report NAM 04, Department of Electrical
%       Engineering and Computer Science, Northwestern University, Evanston,
%       Illinois.

n = size(x,1); % x is assumed to be a column vector;

if prob == 1

  if (n < 6)
     disp('Error: n has to be greater or equal to 7');
     F = 999999;
     return;
  end

  f(1) = x(1)*(2+5*x(1)^2) + 1 - x(2)*(1+x(2));
  f(2) = x(2)*(2+5*x(2)^2) + 1 - x(1)*(1+x(1))-x(3)*(1+x(3));
  f(3) = x(3)*(2+5*x(3)^2) + 1 - sum(x(1:2).*(ones(2,1)+x(1:2)))-x(4)*(1+x(4));
  f(4) = x(4)*(2+5*x(4)^2) + 1 - sum(x(1:3).*(ones(3,1)+x(1:3)))-x(5)*(1+x(5));
  f(5) = x(5)*(2+5*x(5)^2) + 1 - sum(x(1:4).*(ones(4,1)+x(1:4)))-x(6)*(1+x(6));
  f(n) = x(n)*(2+5*x(n)^2) + 1 - sum(x(n-5:n-1).*(ones(5,1)+x(n-5:n-1)));

  for i=6:n-1
    f(i) = x(i)*(2+5*x(i)^2) + 1 - sum(x(i-5:i-1).*(ones(5,1)+x(i-5:i-1)))-x(i+1)*(1+x(i+1));
  end

  F = sum(f.^2);

end

if prob == 2
 
  for i=2:n
    f(i) = 100*(x(i)-x(i-1)^2)^2 + (1-x(i))^2;
  end

  F = sum(f);

end

if prob == 21

  const = 1000;

  for i=2:n
    f(i) = const*(x(i)-x(i-1)^2)^2 + (1-x(i))^2;
  end

  F = sum(f);

end

if prob == 22
 
  for i=2:n
    f(i) = 100*(x(i)-x(i-1))^2 + (1-x(i))^2;
  end

  F = sum(f);

end

if prob == 23

  for i=2:n
    f(i) = 100*(x(i)-x(i-1)^2)^2 + (1-x(i-1))^2;
  end

  F = sum(f);

end

if prob == 3

  for i=1:n
    for j=1:n
       f(i,j) = sin(x(i))^2*sin(x(j))^2*sin(x(i)-x(j))^2;
    end
  end

  F = -sum(sum(f));

end

if prob == 4

  if ((n/4 ~= floor(n/4)) & (n/4 ~= ceil(n/4)))
     disp('Error: n has to be a multiple of 4');
     F = 999999;
     return;
  end

  div = n/4;

  for k=1:div
     f(4*k-3) = x(4*k -3) + 10*x(4*k -2);
     f(4*k-2) = sqrt(5)*(x(4*k -1)-x(4*k));
     f(4*k-1) = (x(4*k-2)-2*x(4*k-1))^2;
     f(4*k)   = sqrt(10)*(x(4*k-3) -x(4*k))^2;
  end

  F = sum(f.^2);

end

if prob == 5

  for i=1:n
     f(i) = 100*(x(1)-x(i)^2)^2+(1-x(i))^2;
  end

  F = sum(f);

end

if prob == 6

  for i=2:n
    f(i) = sqrt(i)*(2*x(i)-x(i-1));
  end

  F = sum(f.^2);

end

if prob == 7

  if ((n/2 ~= floor(n/2)) & (n/2 ~= ceil(n/2)))
     disp('Error: n has to be a multiple of 2');
     F = 999999;
     return;
  end  

  div = n/2;

  for k=1:div
     f(k) = 100*(x(2*k)-x(2*k-1)^2)^2+(1-x(2*k-1))^2;
  end

  F = sum(f);

end

if prob == 8

  if (n<3)
     disp('Error: n has to be greater or equal to 3');
     F = 999999;
     return;
  end

  f(1) = 2*x(1) -x(2) + (x(1) + (n+2)/(n+1))^3/(2*(n+1)^2);
  f(n) = 2*x(n) -x(n-1) + (x(n) + (2*n+1)/(n+1))^3/(2*(n+1)^2);

  for i=2:n-1
     f(i) = 2*x(i) -x(i-1) -x(i+1) +(x(i) + (n+i+1)/(n+1))^3/(2*(n+1)^2); 
  end

  F = sum(f.^2);

end

if prob == 9

  f(1) = (3-2*x(1))*x(1) -2*x(2) +1;
  f(n) = (3-2*x(n))*x(n) -x(n-1) +1;

  for i=2:n-1
     f(i) = (3-2*x(i))*x(i) -x(i-1) -2*x(i+1) +1; 
  end

  F = sum(f.^2);

end

if prob == 10

  if (n~=31)
     disp('Error: n has to be equal to 31');
     F = 999999;
     return;
  end

  index = [1:n-1]';

  f(30) = x(1);
  f(31) = x(2)-x(1)^2-1;
  for i=1:29
    f(i) = sum(index.*x(2:n).*((i/29*ones(n-1,1)).^(index-ones(n-1,1))) )-...
           (sum(x.*((i/29*ones(n,1)).^([0:n-1]'))))^2 - 1;
  end
 
  F = sum(f.^2);

end

if prob == 11

  for i=1:n-1
    f(i) = x(i) + sum(x) - (n+1);
  end
  f(n) = prod(x) -1;
  
  F = sum(f.^2);

end

if prob == 12

  m = 2*n;

  for i=1:n
    f(i) = x(i) -2/m*sum(x) -1;
  end

  for i=n+1:m
    f(i) = -2/m*sum(x) -1;
  end

  F = sum(f.^2);

end

if prob == 13

  m=2*n;

  for i=1:m
    f(i) = i*sum(([1:n]').*x)-1;
  end

  F = sum(f.^2);

end

if prob == 14

  m = 2*n;

  f(1) = -1;
  f(m) = -1;

  for i=2:m-1
    f(i) = (i-1)*sum(([2:n-1]').*x(2:n-1)) -1;
  end

  F = sum(f.^2);

end

if prob == 15

  f(n) = (x(n) - 1)^2;

  for i=1:n-1
    f(i) = (x(i)^2-1)^2;
  end

  F = sum(f);

end

if prob == 16

  if ((n/2 ~= floor(n/2)) & (n/2 ~= ceil(n/2)))
     disp('Error: n has to be a multiple of 2');
     F = 999999;
     return;
  end

  F = 0;
  a = 0.4238537991;

  for i=1:n/2
    F = F+(x(2*i-1)+a)^2*(x(2*i)+a)^2+(x(2*i-1)+a)^2+(x(2*i)+a)^2;
  end

end

if prob == 17
  if ((n/2 ~= floor(n/2)) & (n/2 ~= ceil(n/2)))
     disp('Error: n has to be a multiple of 2');
     F = 999999;
     return;
  end

  F=0;

  for i=1:n/2
    F = F + (exp(x(2*i-1)+x(2*i))-1)^2;
  end
end




























