function [F,G,Hessien] = objgradhess(x,prob)

% This function returns the objective value F(x) of F associated with problem
% number "prob", the gradient and the Hessien (returned in sparse format).
%
% Here are the problems and their related number
%
% 1 = Broyden banded function BRYBND (31) in [1]
% 2 = Generalized Rosenbrock function GENROSE [2]
% 21 = GENROSE, but a scalar of 1000 is used instead in the function
% 22 = a function close to GENROSE (no reference)
% 23 = generalization of the banana function (no reference)
% 3 = Optimal Sensor Placement SENSORS [3]
% 4 = Extended Powell Singular Function PWSING (22) in [1]
% 5 = NONDIA in [4] 
% 6 = TRIDIA in [4]
% 7 = EXTROS in [4]
% 8 = Discrete Boundary Value Function (28) in [1]
% 9= Broyden Tridiagonal Function (30) in [1]
% 10= Watson Function (20) in [1]
% 11= Brown Almost-Linear Function (27) in [1]
%
% Note that for the last 2 problems, 'm' can be variable. It is fixed to 2*n
% in the code right now.
%
% 12= Linear Function-full rank (32) in [1]
% 13= Linear function-rank 1 (33) in [1]
% 14= Linear function-rank 1 with zero columns and rows (34) in [1]
% 15= A function I created so the hard case (case 2) would occur initially
%
% [1] = J.J. More, B.S. Garbow and K.E. Hillstrom, Testing unconstrained
%       optimization software, 7(1981), no 1, pp. 17-41.
%
% [2] = S. Nash, Newton-type minimization via the Lanczos process, SIAM journal
%       on numerical analysis, 21, 1884, pp. 770-788.
%
% [3] = H. Zhang, X. Wang, Optimal Sensor placement, SIAM Review, vol. 35,
%       pp. 641, 1993.
%
% [4] = G. Liu and J. Nocedal, Test results of two limited memory methods for 
%       large scale optimization, Report NAM 04, Department of Electrical
%       Engineering and Computer Science, Northwestern University, Evanston,
%       Illinois.

n = size(x,1); % x is assumed to be a column vector;

if prob == 1

  if (n < 6)
     disp('Error: n has to be greater or equal to 7');
     F = 999999;
     return;
  end

%  J(1,:) = [2,0,0,0,0,0]; 
%  J(2,:) = [1,3,0,0,0,0];
%  J(3,:) = [1,2,4,0,0,0];
%  J(4,:) = [1,2,3,5,0,0];
%  J(5,:) = [1,2,3,4,6,0];
%  J(n,:) = [n-5,n-4,n-3,n-2,n-1,0];
  
%  for i=6:n-1
%    J(i,:) = [i-5,i-4,i-3,i-2,i-1,i+1]; 
%  end

%  for i=1:n
%      somme = 0;
%   for j=1:6
%      if (J(i,j) ~= 0)
%          somme = somme + x(J(i,j))*(1+x(J(i,j)));
%
%      end
%   end
%   f(i) = x(i)*(2+5*x(i)^2) + 1 - somme;
%
%  end
%
%  F = sum(f.^2);

  f(1) = x(1)*(2+5*x(1)^2) + 1 - x(2)*(1+x(2));
  f(2) = x(2)*(2+5*x(2)^2) + 1 - x(1)*(1+x(1))-x(3)*(1+x(3));
  f(3) = x(3)*(2+5*x(3)^2) + 1 - sum(x(1:2).*(ones(2,1)+x(1:2)))-x(4)*(1+x(4));
  f(4) = x(4)*(2+5*x(4)^2) + 1 - sum(x(1:3).*(ones(3,1)+x(1:3)))-x(5)*(1+x(5));
  f(5) = x(5)*(2+5*x(5)^2) + 1 - sum(x(1:4).*(ones(4,1)+x(1:4)))-x(6)*(1+x(6));
  f(n) = x(n)*(2+5*x(n)^2) + 1 - sum(x(n-5:n-1).*(ones(5,1)+x(n-5:n-1)));

  for i=6:n-1
    f(i) = x(i)*(2+5*x(i)^2) + 1 - sum(x(i-5:i-1).*(ones(5,1)+x(i-5:i-1)))-x(i+1)*(1+x(i+1));
  end

  F = sum(f.^2);

%  gradf = zeros(n,n);

  gradf(1,1) = 2+15*x(1)^2;
  gradf(2,1) = 2*x(2) + 1;
  gradf(2,2) = 2+15*x(2)^2;
  gradf(1,2) = 2*x(1) + 1;
  gradf(3,2) = 2*x(3) + 1;
  gradf(3,3) = 2+15*x(3)^2;
  gradf(1,3) = 2*x(1) + 1;
  gradf(2,3) = 2*x(2) + 1;
  gradf(4,3) = 2*x(4) + 1;
  gradf(4,4) = 2+15*x(4)^2;
  gradf(1,4) = 2*x(1) + 1;
  gradf(2,4) = 2*x(2) + 1;
  gradf(3,4) = 2*x(3) + 1;
  gradf(5,4) = 2*x(5) + 1;
  gradf(5,5) = 2+15*x(5)^2;
  gradf(1,5) = 2*x(1) + 1;
  gradf(2,5) = 2*x(2) + 1;
  gradf(3,5) = 2*x(3) + 1;
  gradf(4,5) = 2*x(4) + 1;
  gradf(6,5) = 2*x(6) + 1;
  gradf(n,n) = 2+15*x(n)^2;
  gradf(n-5,n) = 2*x(n-5) + 1;
  gradf(n-4,n) = 2*x(n-4) + 1;
  gradf(n-3,n) = 2*x(n-3) + 1;
  gradf(n-2,n) = 2*x(n-2) + 1;
  gradf(n-1,n) = 2*x(n-1) + 1;

%  for i=1:n
%    gradf2(i,i) = 2+15*x(i)^2;
%    for k=1:6
%      if (J(i,k)~= 0)
%         gradf2(J(i,k),i) = 2*x(J(i,k)) + 1;
%      end
%    end
%  end

  for i=6:n-1
    gradf(i,i) = 2+15*x(i)^2;
    gradf(i-5,i) =  2*x(i-5) + 1;
    gradf(i-4,i) =  2*x(i-4) + 1;
    gradf(i-3,i) =  2*x(i-3) + 1;
    gradf(i-2,i) =  2*x(i-2) + 1;  
    gradf(i-1,i) =  2*x(i-1) + 1;
    gradf(i+1,i) =  2*x(i+1) + 1;
  end

  gradf = sparse(gradf);

  G = 2*gradf*f';

%  H = cell(n,1);

%  for i=1:n
%     H{i} = zeros(n,n);
%     H{i}(i,i) = 16*x(i);
%     for k=1:6
%        if (J(i,k) ~= 0)
%          H{i}(J(i,k),J(i,k))=2;
%        end
%     end
%  end  
    
%  Hessien = zeros(n,n);
%  for i=1:n
%     Hessien = Hessien + 2*(gradf(:,i)*gradf(:,i)' + f(i)*H{i});
%  end

   H1(1,1) = 16*x(1)*f(1)+sum(f(2:6));
   H1(n-4,n-4) = 16*x(n-4)*f(n-4) + 2*sum([f(n-5),f(n-3:n)]);
   H1(n-3,n-3) = 16*x(n-3)*f(n-3) + 2*sum([f(n-4),f(n-2:n)]);
   H1(n-2,n-2) = 16*x(n-2)*f(n-2) + 2*sum([f(n-3),f(n-1:n)]);
   H1(n-1,n-1) = 16*x(n-1)*f(n-1) + 2*sum([f(n-2),f(n)]);
   H1(n,n)     = 16*x(n)*f(n) + 2*f(n-1);

   for i=2:n-5
     H1(i,i) = 16*x(i)*f(i) + 2*sum([f(i-1),f(i+1:i+5)]);
   end
   
   H1 = sparse(H1);

   H2 = gradf(:,1)*gradf(:,1)';
   for i=2:n
     H2 = H2 + gradf(:,i)*gradf(:,i)';
   end

   H2 = sparse(H2);

  Hessien = 2*(H1 + H2); Hessien = sparse(Hessien);

end

if prob == 2
 
  const = 100;

  for i=2:n
    f(i) = const*(x(i)-x(i-1)^2)^2 + (1-x(i))^2;
  end

  F = sum(f);

   G(1) = 4*const*x(1)^3-4*const*x(1)*x(2);
   G(n) = 2*(const+1)*x(n) -2*const*x(n-1)^2 -2;

   for i=2:n-1
      G(i) = 2*(const+1)*x(i) - 2*const*x(i-1)^2-2+4*const*x(i)^3-4*const*x(i)*x(i+1);
   end

   G = G';

   Hessien(1,1) = 12*const*x(1)^2-4*const*x(2);
   Hessien(n,n) = 2*const+2;
   Hessien(n,n-1) = -4*const*x(n-1);
   Hessien(n-1,n) = -4*const*x(n-1);

   Hessien = sparse(Hessien);

   for i=2:n-1
     Hessien(i,i-1) = -4*const*x(i-1);
     Hessien(i-1,i) = -4*const*x(i-1);
     Hessien(i,i) = 2*(1+const)+12*const*x(i)^2-4*const*x(i+1);
   end

   Hessien = sparse(Hessien);

end

if prob == 21
 
  const = 1000;

  for i=2:n
    f(i) = const*(x(i)-x(i-1)^2)^2 + (1-x(i))^2;
  end

  F = sum(f);

   G(1) = 4*const*x(1)^3-4*const*x(1)*x(2);
   G(n) = 2*(const+1)*x(n) -2*const*x(n-1)^2 -2;

   for i=2:n-1
      G(i) = 2*(const+1)*x(i) - 2*const*x(i-1)^2-2+4*const*x(i)^3-4*const*x(i)*x(i+1);
   end

   G = G';

   Hessien(1,1) = 12*const*x(1)^2-4*const*x(2);
   Hessien(n,n) = 2*const+2;
   Hessien(n,n-1) = -4*const*x(n-1);
   Hessien(n-1,n) = -4*const*x(n-1);

   Hessien = sparse(Hessien);

   for i=2:n-1
     Hessien(i,i-1) = -4*const*x(i-1);
     Hessien(i-1,i) = -4*const*x(i-1);
     Hessien(i,i) = 2*(1+const)+12*const*x(i)^2-4*const*x(i+1);
   end

   Hessien = sparse(Hessien);

end

if prob == 22
 
  for i=2:n
    f(i) = 100*(x(i)-x(i-1))^2 + (1-x(i))^2;
  end

  F = sum(f);
  
%  gradf = zeros(n,n);
%  for i=2:n
%    gradf(i,i) = 202*x(i)-200*x(i-1)-2;
%    gradf(i-1,i) = -200*x(i) +200*x(i-1);  
%  end

%  gradf = sparse(gradf);

%  G = sum(gradf')';

%  G = full(G);

   G(1) = -200*x(2) +200*x(1);
   G(n) = 202*x(n) -200*x(n-1) -2;

   for i=2:n-1
      G(i) = 402*x(i) - 200*x(i-1) -200*x(i+1) -2;
   end

   G = G';

%  H = cell(n,1);

%  for i=2:n
%     H{i} = zeros(n,n);
%     H{i}(i,i) = 202;
%     H{i}(i-1,i-1) = 200;
%     H{i}(i,i-1) = -200;
%     H{i}(i-1,i) = -200;
%  end  
   
%  Hessien = zeros(n,n);
%  for i=2:n
%     Hessien = Hessien + H{i};
%  end

%  Hessien = sparse(Hessien);  

   Hessien(1,1) = 200;
   Hessien(n,n) = 202;
   Hessien(n-1,n) = -200;
   Hessien(n,n-1) = -200;

   Hessien = sparse(Hessien);

   for i=2:n-1
     Hessien(i,i-1) = -200;
     Hessien(i-1,i) = -200;
     Hessien(i,i) = 402;
   end

   Hessien = sparse(Hessien);

end

if prob == 23
 
  const = 100;

  for i=2:n
    f(i) = const*(x(i)-x(i-1)^2)^2 + (1-x(i-1))^2;
  end

  F = sum(f);

   G(1) = 4*const*x(1)^3-4*const*x(1)*x(2)-2+2*x(1);
   G(n) = 2*const*x(n) -2*const*x(n-1)^2;

if n>2
   for i=2:n-1
      G(i) = 2*const*x(i) - 2*const*x(i-1)^2-4*const*x(i)*x(i+1)+4*const*x(i)^3-2+2*x(i);
   end
end

   G = G';

   Hessien(1,1) = 12*const*x(1)^2-4*const*x(2)+2;
   Hessien(n,n) = 2*const;
   Hessien(n,n-1) = -4*const*x(n-1);
   Hessien(n-1,n) = -4*const*x(n-1);

   Hessien = sparse(Hessien);

if n>2
   for i=2:n-1
     Hessien(i,i-1) = -4*const*x(i-1);
     Hessien(i-1,i) = -4*const*x(i-1);
     Hessien(i,i) = 2*const+12*const*x(i)^2-4*const*x(i+1)+2;
   end
end

   Hessien = sparse(Hessien);

end

if prob == 3

  for i=1:n
    for j=1:n
       f(i,j) = sin(x(i))^2*sin(x(j))^2*sin(x(i)-x(j))^2;
    end
  end

  F = -sum(sum(f));

  for i=1:n
    G(i) = -2*sum((sin(x)).^2.*(sin(2*x(i))*(sin(x(i)*ones(n,1)-x)).^2 +...
                        (sin(x(i)))^2*sin(2*(x(i)*ones(n,1)-x))));
  end

  G = G';

  for i=1:n
    Hessien(i,i) = -2*sum(sin(x).^2.*(2*cos(2*x(i))*sin(x(i)*ones(n,1)-x).^2 ...
                   +2*sin(2*x(i))*sin(2*(x(i)*ones(n,1)-x))+ ... 
                    2*sin(x(i))^2*cos(2*(x(i)*ones(n,1)-x)))) +...
                    4*sin(x(i))^4;
    for j=i+1:n
       Hessien(i,j) = -2*(sin(2*x(j))*(sin(2*x(i))*(sin(x(i)-x(j)))^2 +...
                      (sin(x(i)))^2*sin(2*(x(i)-x(j)))) + ...
                      (sin(x(j)))^2*(-sin(2*x(i))*sin(2*(x(i)-x(j)))- ...
                      2*(sin(x(i)))^2*cos(2*(x(i)-x(j)))));
       Hessien(j,i) = Hessien(i,j);
    end
  end

  

end

if prob == 4

  if ((n/4 ~= floor(n/4)) & (n/4 ~= ceil(n/4)))
     disp('Error: n has to be a multiple of 4');
     F = 999999;
     return;
  end

  div = n/4;

  for k=1:div
     f(4*k-3) = x(4*k -3) + 10*x(4*k -2);
     f(4*k-2) = sqrt(5)*(x(4*k -1)-x(4*k));
     f(4*k-1) = (x(4*k-2)-2*x(4*k-1))^2;
     f(4*k)   = sqrt(10)*(x(4*k-3) -x(4*k))^2;
  end

  F = sum(f.^2);

  for k=1:div
     gradf(4*k-3,4*k-3) = 1;
     gradf(4*k-2,4*k-3) = 10;
     gradf(4*k-1,4*k-2) = sqrt(5);
     gradf(4*k,4*k-2) = -sqrt(5);
     gradf(4*k-2,4*k-1) = 2*x(4*k-2)-4*x(4*k-1);
     gradf(4*k-1,4*k-1) = 8*x(4*k-1)-4*x(4*k-2);
     gradf(4*k-3,4*k) = sqrt(10)*(2*x(4*k-3)-2*x(4*k));
     gradf(4*k,4*k) = sqrt(10)*(2*x(4*k)-2*x(4*k-3));    
  end

  gradf = sparse(gradf);

  G = 2*gradf*f';
  
  for k=1:div
    H1(4*k-2,4*k-2) = 2*f(4*k-1);
    H1(4*k-2,4*k-1) = -4*f(4*k-1);
    H1(4*k-1,4*k-2) = -4*f(4*k-1);
    H1(4*k-1,4*k-1) = 8*f(4*k-1);
    H1(4*k-3,4*k-3) = 2*sqrt(10)*f(4*k);
    H1(4*k,4*k)     = 2*sqrt(10)*f(4*k);
    H1(4*k-3,4*k)   = -2*sqrt(10)*f(4*k);
    H1(4*k,4*k-3)   = -2*sqrt(10)*f(4*k);
  end

  H1 = sparse(H1);

  H2 = gradf(:,1)*gradf(:,1)';
   for i=2:n
     H2 = H2 + gradf(:,i)*gradf(:,i)';
   end

  H2 = sparse(H2);

  Hessien = 2*(H1 + H2); Hessien = sparse(Hessien); 

end

if prob == 5

  for i=1:n
     f(i) = 100*(x(1)-x(i)^2)^2+(1-x(i))^2;
  end

  F = sum(f);
  
  G(1) = 200*sum(x(1)*ones(n-1,1) - x(2:n));
  for i=2:n
    G(i) = -400*(x(1)-x(i)^2)*x(i)-2*(1-x(i));
  end

  G = G';

  Hessien(1,1) = 200*n;
  for i=2:n
    Hessien(i,1) = -400*x(i);
    Hessien(1,i) = -400*x(i);
    Hessien(i,i) = -400*(x(1)-x(i)^2)+800*x(i)^2+2;
  end

  Hessien = sparse(Hessien);

end

if prob == 6

  for i=2:n
    f(i) = sqrt(i)*(2*x(i)-x(i-1));
  end

  F = sum(f.^2);

  G(1) = -4*(2*x(2)-x(1));
  G(n) = 4*n*(2*x(n)-x(n-1));
  for i=2:n-1
    G(i) = 4*i*(2*x(i)-x(i-1)) - 2*(i+1)*(2*x(i+1)-x(i));
  end

  G = G';

  Hessien(1,1) = 4;
  Hessien(n,n) = 8*n;
  Hessien(n-1,n) = -4*n;
  Hessien(n,n-1) = -4*n;
  for i=2:n-1
    Hessien(i-1,i) = -4*i;
    Hessien(i,i-1) = -4*i;
    Hessien(i,i)   = 8*i+2*(i+1);
  end

  Hessien = sparse(Hessien);

end

if prob == 7

  if ((n/2 ~= floor(n/2)) & (n/2 ~= ceil(n/2)))
     disp('Error: n has to be a multiple of 2');
     F = 999999;
     return;
  end  

  div = n/2;

  for k=1:div
     f(k) = 100*(x(2*k)-x(2*k-1)^2)^2+(1-x(2*k-1))^2;
  end

  F = sum(f);

  for k=1:div
    G(2*k-1) = -400*x(2*k-1)*(x(2*k)-x(2*k-1)^2)-2*(1-x(2*k-1));
    G(2*k) = 200*(x(2*k)-x(2*k-1)^2);
  end

  G = G';

  for k=1:div
    Hessien(2*k-1,2*k-1) = 1200*x(2*k-1)^2-400*x(2*k) + 2;
    Hessien(2*k-1,2*k) = -400*x(2*k-1);
    Hessien(2*k,2*k-1) = -400*x(2*k-1);
    Hessien(2*k,2*k)   = 200;
  end

  Hessien = sparse(Hessien);

end

if prob == 8

  if (n<3)
     disp('Error: n has to be greater or equal to 3');
     F = 999999;
     return;
  end

  f(1) = 2*x(1) -x(2) + (x(1) + (n+2)/(n+1))^3/(2*(n+1)^2);
  f(n) = 2*x(n) -x(n-1) + (x(n) + (2*n+1)/(n+1))^3/(2*(n+1)^2);

  for i=2:n-1
     f(i) = 2*x(i) -x(i-1) -x(i+1) +(x(i) + (n+i+1)/(n+1))^3/(2*(n+1)^2); 
  end

  F = sum(f.^2);

  gradf(1,1) = 2 + 3/(2*(n+1)^2)*(x(1)+(n+2)/(n+1))^2;
  gradf(2,1) = -1;
  gradf(n,n) = 2 + 3/(2*(n+1)^2)*(x(n)+(2*n+1)/(n+1))^2;
  gradf(n-1,n) = -1;
  for i=2:n-1
    gradf(i-1,i) = -1;
    gradf(i,i)   = 2+3/(2*(n+1)^2)*(x(i)+(n+i+1)/(n+1))^2;
    gradf(i+1,i) = -1;
  end

  gradf = sparse(gradf);

  G = 2*gradf*f';

  for i=1:n
    H1(i,i) = f(i)*(x(i)+(n+i+1)/(n+1));
  end
  H1 = 3/(n+1)^2*H1;

  H1 = sparse(H1);

  H2 = gradf(:,1)*gradf(:,1)';
  for i=2:n
     H2 = H2 + gradf(:,i)*gradf(:,i)';
  end

  H2 = sparse(H2);

  Hessien = 2*(H1 + H2); Hessien = sparse(Hessien);  

end

if prob == 9

  f(1) = (3-2*x(1))*x(1) -2*x(2) +1;
  f(n) = (3-2*x(n))*x(n) -x(n-1) +1;

  for i=2:n-1
     f(i) = (3-2*x(i))*x(i) -x(i-1) -2*x(i+1) +1; 
  end

  F = sum(f.^2);

  gradf(1,1) = -4*x(1)+3;
  gradf(2,1) = -2;
  gradf(n-1,n) = -1;
  gradf(n,n) = -4*x(n)+3;
  for i=2:n-1
    gradf(i-1,i) = -1;
    gradf(i,i)   = -4*x(i)+3;
    gradf(i+1,i) = -2;
  end
 
  gradf = sparse(gradf);

  G = 2*gradf*f';

  for i=1:n
    H1(i,i) = f(i);
  end
  H1 = -4*H1;

  H1 = sparse(H1);

  H2 = gradf(:,1)*gradf(:,1)';
  for i=2:n
     H2 = H2 + gradf(:,i)*gradf(:,i)';
  end

  H2 = sparse(H2);

  Hessien = 2*(H1 + H2); Hessien = sparse(Hessien);  

end

if prob == 10

  if (n~=31)
     disp('Error: n has to be equal to 31');
     F = 999999;
     return;
  end

  index = [1:n-1]';

  f(30) = x(1);
  f(31) = x(2)-x(1)^2-1;
  for i=1:29
    f(i) = sum(index.*x(2:n).*((i/29*ones(n-1,1)).^(index-ones(n-1,1))) )-...
           (sum(x.*((i/29*ones(n,1)).^([0:n-1]'))))^2 - 1;
  end
 
  F = sum(f.^2);

%  for i=1:29
%    gradf(1,i) = -2*x(1) - 2*sum(x.*((i/29*ones(n,1)).^([0:n-1]'))) + 2*x(1);
%    for j=2:31
%      gradf(j,i) = (i-1)*i/29^(i-2) - ...
%                   2*x(i)*((i/29)^(i-1))^2 - ...
%                   2*sum((i/29)^(i-1)*x.*((i/29*ones(n,1)).^([0:n-1]'))) +...
%                   2*x(i)*((i/29)^2)^(i-1);
%    end
%  end
%
%  gradf(1,30) = 1;
%  gradf(1,31) = -2*x(1);
%  gradf(2,31) = 1;

   for i=1:29
    g1 =0;
    for j=2:n
     g1 = g1+x(j)*(i/29)^(j-1);
    end
    gradf(1,i) = -2*x(1)-2*g1;
    for k=2:n
     g2 = 0;
     for h=1:n
       g2 = g2+x(h)*(i/29)^(k-1)*(i/29)^(h-1);
     end
     g2 = g2 - x(k)*(i/29)^(k-1)*(i/29)^(k-1);
     gradf(k,i) = (k-1)*(i/29)^(k-2)-2*x(k)*((i/29)^(k-1))^2 -2*g2;
    end
 
   end

   gradf(1,30) = 1;
   gradf(1,31) = -2*x(1);
   gradf(2,31) = 1;

  G = 2*gradf*f';

  for i=1:n
   H1(i,i) = -2*((1/29)^(i-1))^2;
   for j=i+1:n
     H1(i,j) = -2*(1/29)^(i+j-2);
     H1(j,i) = H1(i,j);
   end
  end
  H1 = f(1)*H1;
  
  for k=2:29
    
    for i=1:n
     Htemp(i,i) = -2*((k/29)^(i-1))^2;
     for j=i+1:n
       Htemp(i,j) = -2*(k/29)^(i+j-2);
       Htemp(j,i) = Htemp(i,j);
     end
    end    

  Htemp = f(k)*Htemp;
  H1 = H1+Htemp;
  end

  H1(1,1) = H1(1,1)-2*f(31);

  H2 = gradf(:,1)*gradf(:,1)';
  for i=2:n
     H2 = H2 + gradf(:,i)*gradf(:,i)';
  end

  Hessien = 2*(H1+H2);

end

if prob == 11

  for i=1:n-1
    f(i) = x(i) + sum(x) - (n+1);
  end
  f(n) = prod(x) -1;
  
  F = sum(f.^2);

  gradf = ones(n,n-1);

  for i=1:n-1
    gradf(i,i) = 2;
  end

  gradf(1,n) = prod(x(2:n));
  gradf(n,n) = prod(x(1:n-1));

  for i=2:n-1
    d = [1:i-1,i+1:n];
    gradf(i,n) = prod(x(d));
  end  

  G = 2*gradf*f';

  for i=1:n-1
   for j=i+1:n
     d = [1:i-1,i+1:j-1,j+1:n];
     H1(i,j) = prod(x(d));
     H1(j,i) = H1(i,j);
   end
  end

  H1 = f(n)*H1;

  H2 = gradf(:,1)*gradf(:,1)';
  for i=2:n
     H2 = H2 + gradf(:,i)*gradf(:,i)';
  end

  Hessien = 2*(H1+H2);  

end

if prob == 12

  m = 2*n;

  for i=1:n
    f(i) = x(i) -2/m*sum(x) -1;
  end

  for i=n+1:m
    f(i) = -2/m*sum(x) -1;
  end

  F = sum(f.^2);

  for i=1:n
   gradf(:,i) = -2/m*ones(n,1); gradf(i,i) = 1 + gradf(i,i);
  end

  for i=n+1:m
   gradf(:,i) = -2/m*ones(n,1);
  end

  G = 2*gradf*f';

  Hessien = gradf(:,1)*gradf(:,1)';
  for i=2:m
     Hessien = Hessien + gradf(:,i)*gradf(:,i)';
  end

end

if prob == 13

  m=2*n;

  for i=1:m
    f(i) = i*sum(([1:n]').*x)-1;
  end

  F = sum(f.^2);

  for i=1:m
    gradf(:,i) = i*[1:n]';
  end

  G = 2*gradf*f';

  Hessien = gradf(:,1)*gradf(:,1)';
  for i=2:m
     Hessien = Hessien + gradf(:,i)*gradf(:,i)';
  end

end

if prob == 14

  m = 2*n;

  f(1) = -1;
  f(m) = -1;

  for i=2:m-1
    f(i) = (i-1)*sum(([2:n-1]').*x(2:n-1)) -1;
  end

  F = sum(f.^2);

  gradf(:,1) = zeros(n,1);
  gradf(:,m) = zeros(n,1);

  for i=2:m-1
    gradf(:,i) = (i-1)*[1:n]';
    gradf(1,i) = 0;
    gradf(n,i) = 0;
  end

  G = 2*gradf*f';


  Hessien = gradf(:,1)*gradf(:,1)';
  for i=2:m
     Hessien = Hessien + gradf(:,i)*gradf(:,i)';
  end

end

if prob == 15

  f(n) = (x(n) - 1)^2;

  for i=1:n-1
    f(i) = (x(i)^2-1)^2;
  end

  F = sum(f);

  G(n) = 2*(x(n)-1);

  for i=1:n-1
    G(i) = 4*x(i)*(x(i)^2-1);
  end

  G = G';  

  Hessien(n,n) = 2;

  for i=1:n-1
    Hessien(i,i) = 12*x(i)^2-4;
  end

  Hessien = sparse(Hessien);

end

if prob == 16

  if ((n/2 ~= floor(n/2)) & (n/2 ~= ceil(n/2)))
     disp('Error: n has to be a multiple of 2');
     F = 999999;
     return;
  end

  F = 0;
  a = 0.4238537991;

  for i=1:n/2
    F = F+(x(2*i-1)+a)^2*(x(2*i)+a)^2+(x(2*i-1)+a)^2+(x(2*i)+a)^2;
  end

  for i=1:n/2
    G(2*i-1) = 2*(x(2*i-1)+a)*((x(2*i)+a)^2+1);
    G(2*i  ) = 2*(x(2*i)+a)*((x(2*i-1)+a)^2+1);
  end

  G = G';

  for i=1:n/2
    Hessien(2*i-1,2*i-1) = 2*((x(2*i)+a)^2+1);
    Hessien(2*i-1,2*i)   = 4*(x(2*i-1)+a)*(x(2*i)+a);
    Hessien(2*i,2*i-1)   = 4*(x(2*i-1)+a)*(x(2*i)+a);
    Hessien(2*i,2*i)     = 2*((x(2*i-1)+a)^2+1);
  end

  Hessien = sparse(Hessien);

end

if prob == 17
   if ((n/2 ~= floor(n/2)) & (n/2 ~= ceil(n/2)))
     disp('Error: n has to be a multiple of 2');
     F = 999999;
     return;
  end

  F=0;

  for i=1:n/2
    F = F + (exp(x(2*i-1)+x(2*i))-1)^2;
    G(2*i-1) = 2*(exp(x(2*i-1)+x(2*i))-1)*exp(x(2*i-1)+x(2*i));
    G(2*i) = G(2*i-1);
    Hessien(2*i-1,2*i-1) = 2*exp(x(2*i-1)+x(2*i))*(2*exp(x(2*i-1)+x(2*i))-1);
    Hessien(2*i,2*i) = Hessien(2*i-1,2*i-1);
    Hessien(2*i,2*i-1) = Hessien(2*i-1,2*i-1);
    Hessien(2*i-1,2*i) = Hessien(2*i-1,2*i-1);
  end 

  G = G';
  Hessien = sparse(Hessien);


end